# Generated by Django 4.1.5 on 2023-01-26 10:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0004_alter_postnotificationrule_is_published_prev'),
    ]

    operations = [
        migrations.RenameField(
            model_name='postnotificationreminder',
            old_name='template_prefix',
            new_name='template',
        ),
        migrations.RenameField(
            model_name='postnotificationrule',
            old_name='template_prefix',
            new_name='template',
        ),
        migrations.AlterField(
            model_name='postnotificationreminder',
            name='action',
            field=models.TextField(blank=True, default='', help_text='Python code to execute. You can use `instance` variable to access current model'),
        ),
    ]
