# Generated by Django 4.1.5 on 2023-01-18 05:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0003_postnotificationreminder_action'),
    ]

    operations = [
        migrations.AlterField(
            model_name='postnotificationrule',
            name='is_published_prev',
            field=models.BooleanField(default=False, null=True),
        ),
    ]
